#include <pic.h>
#include <stdio.h>
#define XTAL_FREQ   12MHZ
__CONFIG(0x3F32);

#include "delay.c"
#include "ldisp.h"
#include "effects.h"
#include "uart.h"

void pixel_prep(void);
void build_image(void);
void device_config(void);
void calc_rpm(void);


/*interrupt service routine**/
static void interrupt
isr(void)
{
	unsigned char tempint;
/*TIMER1 MATCH THE CCP2 VALUE, COMPARE INTERRUPT*/
	if(CCP2IE&&CCP2IF)
	{
		do
		{
			CCP2IF=0;
			/*prepare the data for the next pixels*/
			if(bit_test(STARE,MIRROR)) // if the display must be mirored
				current_pix--;
			CCPR2+=pixel_deg; // the angle for the next pixel
			while(!CCP2IF); // wait for the moment of the next pixel
			if(bit_test(STARE,MIRROR)) // if the display must be mirored
			{
				if(current_pix==255)
					break;
			}
			/*prepare the next pixel*/
#if NR_PIXELI<=84
				PORTBM=ldimage1[current_pix]&current_line;
#else
			if(current_pix<=95)
				PORTBM=ldimage1[current_pix]&current_line;
			else
				PORTBM=ldimage2[current_pix-96]&current_line;
#endif
/*************************************/
			bit_set(CCP2CON,0); // clear output on match (the pixel is off)
			if(PORTBM)
				bit_clear(CCP2CON,0); // set output on match (the pixel is on)
/************************************/
			if(!bit_test(STARE,MIRROR))
				current_pix++; // increment the index of pixels
/***********************************/
		}while(((current_pix<=NR_PIXELI)&&!bit_test(STARE,MIRROR)) || (bit_test(STARE,MIRROR)));
		CCP2IE=0;
		CCP2IF=0;
		CCP2CON=0; // torn off the light beam
		current_line=current_line>>1;
		pixel_prep();
	}
/*CAPTURE ON RC2 INTERRUPT used for
measuring of the rotational period of the motor*/
	if(CCP1IF)
	{
		period_b3=0;
		period_b2=count;
		TMR1ON=0; // stop TMR1 
		/*take the value of TMR1*/
		period_b1=TMR1H;
		period_b0=TMR1L;
		/*initialization of TMR1*/
		TMR1L=27;
		TMR1H=0;
		TMR1ON=1;// re enable TMR1
		count=0; // clear the extension of TMR1
		period-=20; // a small corection to the measured period
		CCP1IF=0; // clear the flag
		current_line=0x80;
	/*if PERIOD>=65536 and PERIOD <=262144
	for a nominal rpm of  2020 RPM PERIOD is 148514*/
		if(period_msi>=1 && period_msi<=4)
		{
			tempint=period/DIVIDER;
	/*this is for suppresing of glitches (one bit error)*/
			if(((tempint-1)>pixel_deg) || ((tempint+1)<pixel_deg))
				pixel_deg=tempint;
			ini_deg=period/64; // 11.25 degrees, the necessary time for prepariong the first data
			pixel_prep();
			if(EEPROM_READ(0x10)&(1<<MIRROR))
				bit_set(STARE,MIRROR);
		}
	}
/*TIMER1 OVERFLOW*/
	if(TMR1IF)
	{
		count+=1; // the extension of TMR1
		TMR1IF=0;
	}
	if(INTF)
	{
		INTF=0;
	}
if(RBIF && RBIE)
{
	// debounce
	for(portb_mirror=250; portb_mirror; portb_mirror--)
	{
		asm("	nop");
		asm("	nop");
	}
	if ((PORTB & 0xf0) == 0xf0) // all the keys are unpressed?...
	{
		/*take a decision: which key was pressed?*/
		if(!bit_test(ex_portb,UP) && !bit_test(ex_portb,DOWN))
		{
			/*was UP&DOWN*/
			if(!bit_test(STARE,SET_LINE))
			{
				bit_set(STARE,SET_LINE); // we have to adjust the line position
				setted_line=7; // the first line is 7
			}
			else
			{
				if(setted_line==1)
					bit_clear(STARE,SET_LINE);
				setted_line--; // next line
			}
			ex_portb=0xf0;
			goto EndIntRB;
		}
	  	if(!bit_test(ex_portb,UP))
   	{
   		/*was UP*/
			if(bit_test(STARE,SET_LINE)) // line position adjust?
			{
				portb_mirror=EEPROM_READ(setted_line)+1; // read the delay for the current line and increment
				EEPROM_WRITE(setted_line, portb_mirror); // save the delay
			}
			/*tratarea tastei pana aici*/
			ex_portb=0xf0;
			goto EndIntRB;
   	}
   	if(!bit_test(ex_portb,DOWN))
   	{
   		/*was DOWN*/
			if(bit_test(STARE,SET_LINE)) // line position adjust?
			{
				portb_mirror=EEPROM_READ(setted_line)-1; // read the delay for the current line and decrement
				EEPROM_WRITE(setted_line, portb_mirror); // save the delay
			}
			ex_portb=0xf0;
			goto EndIntRB;
		}
		if(!bit_test(ex_portb,MODE) && !bit_test(ex_portb,SHIFT))
		{ /*was MODE + SHIFT*/
			// display the RPM and the vertical markers for line adjustment
			STARE^=1<<MIRA;
			tmr0ext=0;
			T0IF=0;
			bit_set(STARE,NEWDATA);
			ex_portb=0xf0;
			goto EndIntRB;
		}
		if(!bit_test(ex_portb,MODE))
	   	{ // switch between NORMAL and MIRORED display
			STARE^=1<<MIRROR;
			EEPROM_WRITE(0x10,(STARE&1<<MIRROR));
			current_line=0;
			ex_portb=0xf0;
			goto EndIntRB;
		}
		if(!bit_test(ex_portb,SHIFT))
	   	{ // freeze the display
			STARE^=1<<EFECT_BLOCK;
		/*tratarea tastei pana aici*/
			ex_portb=0xf0;
			goto EndIntRB;
		}
	}
	if((PORTB & 0xf0) != 0xf0) // if at least one key is pressed
	{
		ex_portb&=PORTB; // take the pressed key into ex_portb
	}
EndIntRB:
			portb_mirror=0xF0&PORTB;	// clear missmatch condition
			RBIF=0;	// clear interrupt flag
}
if(RCIF&&RCIE)
{
	portb_mirror=RCREG;
	if(portb_mirror==INICOM)
	{
		bit_set(STARE,PC_COMM);
		TMR1ON=0; // by stopping the TMR1, any measurement or display is blocked
	}
}
}// end of ISR

void main()
{
	for(tmr0ext=8;tmr0ext;tmr0ext--)
	{
		DelayMs(250);
	}
	/*wait for stabilizing of the rotational speed of the motor*/
	device_config();
	nextcar=&show[0]; // initialize the base addres of the show
	while(TRUE)
	{
		if(bit_test(STARE,PC_COMM))
		{
			RCIE=0; // stop the SPI interrupts
			putbyte(ACKCOM); // send the acknowledge
			ShowDownload();// downloat the show
			period_msi=0; // force a period measurement before displaying the data
			ClrScr(); // Clear screen
			nextcar=&show[0]; // reinitialize the base addres of the show
			EFCONFIG=0; // reinitialize the effects configuration
			TMR1ON=1; // start the Timer1
			RCIE=1; // enable the SPI interrups

			bit_clear(STARE,PC_COMM);
		}
		if(bit_test(STARE,NEWDATA)&&bit_test(STARE,EFFECT))
		{
			/*
			it was a togle between rpm and normal display or
			the normal displaying is initialized or the new rpm value
			must be displayed
			*/
			if(bit_test(STARE,MIRA))
			{
				calc_rpm();
				nextcar=&disp_mirror[0];
				EFCONFIG=0;
				build_image();
			}
			else
			{
				ClrScr();
				nextcar=&show[0];
			}
			bit_clear(STARE,NEWDATA);
		}
		if(T0IF)
		{ // once at 13,1ms for a 20MHz clock
			/*
			TMR0 was used to decide when to calculate the RPM for displaying
			*/
			tmr0ext++;
			if(bit_test(tmr0ext,7)&& bit_test(STARE,MIRA)) // 0.6 sec
			{
				bit_set(STARE,NEWDATA);
				tmr0ext=0;
			}
			T0IF=0;
		}
/************************************************************/
		if(bit_test(STARE,EFFECT))
		{/*a new step of the current effect?*/
			switch(*nextcar)
			{
				case NOEF: case TOPR: case BOTR: case RLF: case RLFC: case TBFC:
				{
					EFCONFIG=(EFCONFIG&0xf0)|*nextcar;
					steps=0;
					movedcars=0;
					nextcar++;
					break;
				}
				case EOSH: // end of show show?
				{
					nextcar=&show[0]; // reinitialize the base addres of the show
					break;
				}
				default:	break;
			}
			switch(EFCONFIG&0x0f)
			{
				case NOEF:	case BOTR:	case TOPR:
				{// display the data with a simple effect
					simple_effect();
					break;
				}
				case RLF:{	rlflow();	break;	}
				case RLFC:{	rlchars();	break;	}
				case TBFC:{	fallchar();	break;	}
				default:	break;
			}
			bit_clear(STARE,EFFECT);
		}
/******************************************************/
	};
}

void device_config(void)
{
/*PERIPHERAL SETUP*/
	PORTA=0;
	PORTB=0;
	PORTC=0;
	PORTD=0;
	PORTE=0;
ADCON1=0b00000110;  // all digital
	TRISA=0x1; // all outputs
	TRISB=0xf0; // all otputs except RB0,RB1,RB2,RB3
	TRISC=0b11000100; // RC7,RC6 input for USART RC2 =input, RC1=out (for period meas and laser drive)
	TRISD=0;
	TRISE=0;
/***********************/
	OPTION=0b01010111; // pull-ups enabled, PS to TMR0, rate 1:256, internal clock , inctrement by falling edge
/*CAPTURE,COMPARE SETUP*/
	CCP1CON=0b00000101; // capture mode, every rising edge
	CCP2CON=0b00001001; // compare mode, generate SW interrupt on match and clear CCP2 output
/*TIMER1 SETUP*/
	TMR1H=0;
	TMR1L=50;
	T1CON=0b00000000; // prescaller 0:0, osc=off, internal CLK, disabled
/*INTERRUPTS*/
	INTCON=0b11001000; // GIE, PEIE,RBIE, enabled
	PIE1=0b00000101; // CCP1IE & TMR1IE enabled
/*serial communication setup*/
InitComms();

	ex_portb=0xf0;
}
void pixel_prep(void)
{
/*
this function sets the CCP2 module and calculates the initial
value of CCP2 taking in consideration the calibration values
*/
unsigned char dl_temp1;
unsigned long current_deg;
#define current_deg_b0   (((ulng *)&current_deg)->b0)  // LSB
#define current_deg_b1   (((ulng *)&current_deg)->b1)
#define current_deg_b2   (((ulng *)&current_deg)->b2)
#define current_deg_b3   (((ulng *)&current_deg)->b3) // MSB
#define dl_temp2 current_deg_b0
/**************END OF DECLARATIONS************/
	if(current_line>1)
	{
		dl_temp1=0;
		dl_temp2=current_line;
		while(dl_temp2) // beacuse the current_line is walking 1, I have to calculate the line number
		{
			dl_temp2=dl_temp2>>1;
			dl_temp1++;
		}
		dl_temp1--;
		dl_temp2=EEPROM_READ(dl_temp1);
	/*compensation of a optical ilusion durring text flowing*/
		if((EFCONFIG&0x0f)==RLF && !bit_test(STARE,EFECT_BLOCK))
		{
			if(bit_test(STARE,MIRROR))
				dl_temp2+=8-dl_temp1;
			else
				dl_temp2+=dl_temp1;
		}
/**********************************/
		current_deg=period*(unsigned long)(dl_temp2);
		current_deg/=0x4000;
		current_deg+=ini_deg; //+current_deg/0x4000;
		ini_deg+=period/8; // initial angle
		CCPR2L=current_deg_b0;		CCPR2H=current_deg_b1;
		CCP2CON=0b00001001; // compare mode, generate SW interrupt on match and clear CCP2 output
		CCP2IF=0;	CCP2IE=1;
		PORTBM=0x0;
		if(bit_test(STARE,MIRROR))
			current_pix=NR_PIXELI+1;
		else
			current_pix=0;
	}
	else
	{
		CCP2IE=0;
/****if is not PAUSE, set the flag for the next step of current effect**********/
		if(!bit_test(STARE,EFECT_BLOCK))
			bit_set(STARE,EFFECT);
	}
/************************************/
}
void build_image(void)
{
/*
takes the characters beginning with the character pointed by the nextcar
and bulds the binary image of the display in the ldimage string
*/
	unsigned char cur_car, cur_line; // index for current char and current line
	unsigned char discar; // the char to be displayed corresponding to cur_car index
	unsigned char cur_pix; // index for the current pixel in ldimage string

	for(cur_car=0;cur_car<NR_CARS;cur_car++)
	{
		discar=*nextcar;
		if(discar==0 || discar>127) //bad chars? (error)
		{
			discar=' ';
			break;
		}
		else
			nextcar++;
		discar-=0x20;
		cur_pix=cur_car*6;
		cur_pix+=1;
		for(cur_line=0;cur_line<6;cur_line++)
		{
			if(cur_line!=5)
			{
#if NR_PIXELI<=84
				ldimage1[cur_pix]=char_map[discar][cur_line];
#else
				if(cur_pix<=95)
					ldimage1[cur_pix]=char_map[discar][cur_line];
				else
					ldimage2[cur_pix-96]=char_map[discar][cur_line];
#endif
			}
			else
			{
#if NR_PIXELI<=84
				ldimage1[cur_pix]=0; // 1 pixel for char spacing
#else
				if(cur_pix<=95)
					ldimage1[cur_pix]=0; // 1 pixel for char spacing
				else
					ldimage2[cur_pix-96]=0;
#endif
			}
			cur_pix++;
		}
	}
}
void calc_rpm(void)
{
	unsigned int rpm;
	// rpm calculation (from period)
	rpm=(unsigned int)(300000000UL/period);
// the message will be placed in "disp_mirror"
	sprintf(&disp_mirror[0],"|=Turatia: %urpm=|",rpm);
}


#include "effects.c"
#include "defshow.c"
#include "uart.c"
